#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.component.gallery.interfaces import IGalleryContainerTarget, IGallery, IGalleryContainer, IGalleryFile, \
    IGalleryFileInfo
from onf_website.component.gallery.zmi.interfaces import IGalleryImageAddFields
from pyams_file.interfaces.archive import IArchiveExtractor
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer

# import packages
from onf_website.component.gallery import Gallery
from onf_website.component.gallery.zmi.container import GalleryContainerView
from pyams_file.file import get_magic_content_type, FileFactory
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_form.security import ProtectedFormObjectMixin
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarAction
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field
from zope.interface import alsoProvides
from zope.lifecycleevent import ObjectCreatedEvent

from onf_website import _


@viewlet_config(name='add-gallery.menu', context=IGalleryContainerTarget, view=GalleryContainerView,
                layer=IPyAMSLayer, manager=IWidgetTitleViewletManager, weight=50)
class GalleryAddMenu(ProtectedFormObjectMixin, ToolbarAction):
    """Gallery add menu"""

    label = _("Add gallery")

    url = 'add-gallery.html'
    modal_target = True


@pagelet_config(name='add-gallery.html', context=IGalleryContainerTarget, layer=IPyAMSLayer,
                permission='onf.ManageContent')
class GalleryAddForm(AdminDialogAddForm):
    """Gallery add form"""

    legend = _("Add new images gallery")
    icon_css_class = 'fa fa-fw fa-picture-o'

    fields = field.Fields(IGallery).omit('__parent__', '__name__') + \
             field.Fields(IGalleryImageAddFields)

    @property
    def ajax_handler(self):
        origin = self.request.params.get('origin')
        if origin == 'link':
            return 'add-gallery-link.json'
        else:
            return 'add-gallery.json'

    edit_permission = 'onf.ManageContent'

    def updateWidgets(self, prefix=None):
        super(GalleryAddForm, self).updateWidgets(prefix)
        self.widgets['description'].label_css_class = 'textarea'
        self.widgets['author_comments'].label_css_class = 'textarea'

    def create(self, data):
        gallery = Gallery()
        images = data['images_data']
        if images:
            medias = []
            if isinstance(images, (list, tuple)):
                images = images[1]
            if hasattr(images, 'seek'):
                images.seek(0)
            registry = self.request.registry
            content_type = get_magic_content_type(images)
            if hasattr(images, 'seek'):
                images.seek(0)
            extractor = query_utility(IArchiveExtractor, name=content_type.decode())
            if extractor is not None:
                extractor.initialize(images)
                for content, filename in extractor.get_contents():
                    media = FileFactory(content)
                    registry.notify(ObjectCreatedEvent(media))
                    medias.append(media)
            else:
                media = FileFactory(images)
                registry.notify(ObjectCreatedEvent(media))
                medias.append(media)
            for media in medias:
                alsoProvides(media, IGalleryFile)
                IGalleryFileInfo(media).author = data.get('author')
                IGalleryFileInfo(media).author_comments = data.get('author_comments')
                gallery['none'] = media
        return gallery

    def update_content(self, content, data):
        content.title = data.get('title')
        content.description = data.get('description')
        content.visible = data.get('visible')

    def add(self, object):
        IGalleryContainer(self.context)['none'] = object


@view_config(name='add-gallery.json', context=IGalleryContainerTarget, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class GalleryAJAXAddForm(AJAXAddForm, GalleryAddForm):
    """Gallery add form, JSON renderer"""

    def get_ajax_output(self, changes):
        return {'status': 'reload',
                'location': '#galleries.html'}


@view_config(name='add-gallery-link.json', context=IGalleryContainerTarget, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class GalleryLinkAJAXAddForm(AJAXAddForm, GalleryAddForm):
    """Gallery link add form, JSON renderer"""

    def get_ajax_output(self, changes):
        target = get_parent(self.context, IGalleryContainerTarget)
        container = IGalleryContainer(target)
        galleries = [{'id': gallery.__name__,
                      'text': II18n(gallery).query_attribute('title', request=self.request)}
                     for gallery in container.values()]
        return {'status': 'callback',
                'callback': 'ONF.Skin.galleries.refresh',
                'options': {'galleries': galleries,
                            'new_gallery': {'id': changes.__name__,
                                            'text': II18n(changes).query_attribute('title', request=self.request)}}}


@pagelet_config(name='properties.html', context=IGallery, layer=IPyAMSLayer, permission='system.view')
class GalleryPropertiesEditForm(AdminDialogEditForm):
    """Gallery properties edit form"""

    legend = _("Update gallery properties")
    icon_css_class = 'fa fa-fw fa-picture-o'

    fields = field.Fields(IGallery).omit('__parent__', '__file__')
    ajax_handler = 'properties.json'
    edit_permission = 'onf.ManageContent'

    def updateWidgets(self, prefix=None):
        super(GalleryPropertiesEditForm, self).updateWidgets(prefix)
        self.widgets['description'].label_css_class = 'textarea'


@view_config(name='properties.json', context=IGallery, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class GalleryPropertiesAJAXEditForm(AJAXEditForm, GalleryPropertiesEditForm):
    """Gallery properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        if 'title' in changes.get(IGallery, ()):
            return {'status': 'reload',
                    'location': '#external-files.html'}
        else:
            return super(GalleryPropertiesAJAXEditForm, self).get_ajax_output(changes)
