#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.base.interfaces import IBaseContent
from onf_website.component.location.interfaces import ILocationInfo
from onf_website.root.interfaces import ISiteRootToolsConfiguration
from pyams_catalog.interfaces import DATE_RESOLUTION
from pyams_utils.interfaces.site import ISiteGenerations
from pyams_utils.interfaces.traversing import IPathElements
from pyams_workflow.interfaces import IWorkflowState
from zope.dublincore.interfaces import IZopeDublinCore
from zope.site.interfaces import INewLocalSite

# import packages
from hypatia.text.lexicon import Lexicon
from onf_website.reference import ReferencesManager
from onf_website.reference.location.agency import AgencyTable
from onf_website.reference.location.country import CountryTable
from onf_website.reference.location.department import DepartmentTable
from onf_website.reference.location.region import RegionTable
from onf_website.reference.location.territory import TerritoryTable
from onf_website.reference.location.unit import UnitTable
from onf_website.reference.target import TargetTable
from onf_website.shared.common.manager import SharedToolContainer
from onf_website.shared.news.manager import NewsManager
from pyams_catalog.index import FieldIndexWithInterface, KeywordIndexWithInterface, DatetimeIndexWithInterface
from pyams_catalog.nltk import NltkFullTextProcessor
from pyams_catalog.site import check_required_indexes
from pyams_i18n.index import I18nTextIndexWithInterface
from pyams_security.index import PrincipalsRoleIndex
from pyams_utils.registry import utility_config
from pyams_utils.site import check_required_utilities
from pyramid.events import subscriber
from pyramid.threadlocal import get_current_registry
from zope.lifecycleevent import ObjectCreatedEvent


def get_fulltext_lexicon():
    return Lexicon(NltkFullTextProcessor(language='french'))


REQUIRED_UTILITIES = ()


REQUIRED_INDEXES = (('content_type', FieldIndexWithInterface, {'interface': IBaseContent,
                                                               'discriminator': 'content_type'}),
                    ('owner', FieldIndexWithInterface, {'interface': IBaseContent,
                                                        'discriminator': 'owner'}),
                    ('parents', KeywordIndexWithInterface, {'interface': IPathElements,
                                                            'discriminator': 'parents'}),
                    ('title:fr', I18nTextIndexWithInterface, {'language': 'fr',
                                                              'interface': IBaseContent,
                                                              'discriminator': 'title',
                                                              'lexicon': get_fulltext_lexicon}),
                    ('countries_index', KeywordIndexWithInterface, {'interface': ILocationInfo,
                                                                    'discriminator': 'countries_index'}),
                    ('departments_index', KeywordIndexWithInterface, {'interface': ILocationInfo,
                                                                      'discriminator': 'departments_index'}),
                    ('structures_index', KeywordIndexWithInterface, {'interface': ILocationInfo,
                                                                     'discriminator': 'structures'}),
                    ('workflow_state', FieldIndexWithInterface, {'interface': IWorkflowState,
                                                                 'discriminator': 'state'}),
                    ('role:pilot', PrincipalsRoleIndex, {'role_id': 'onf.Pilot'}),
                    ('role:manager', PrincipalsRoleIndex, {'role_id': 'onf.Manager'}),
                    ('role:contributor', PrincipalsRoleIndex, {'role_id': 'onf.Contributor'}),
                    ('created_date', DatetimeIndexWithInterface, {'interface': IZopeDublinCore,
                                                                  'discriminator': 'created',
                                                                  'resolution': DATE_RESOLUTION}),
                    ('modified_date', DatetimeIndexWithInterface, {'interface': IZopeDublinCore,
                                                                   'discriminator': 'modified',
                                                                   'resolution': DATE_RESOLUTION}))


@subscriber(INewLocalSite)
def handle_new_local_site(event):
    """Check for required utilities when a site is created"""
    site = event.manager.__parent__
    check_required_utilities(site, REQUIRED_UTILITIES)
    check_required_indexes(site, REQUIRED_INDEXES)


@utility_config(name='ONF website', provides=ISiteGenerations)
class WebsiteGenerationsChecker(object):
    """ONF website generations checker"""

    generation = 1

    def evolve(self, site, current=None):
        """Check for required utilities"""
        check_required_utilities(site, REQUIRED_UTILITIES)
        check_required_indexes(site, REQUIRED_INDEXES)
        registry = get_current_registry()
        tools_configuration = ISiteRootToolsConfiguration(site)
        # check references tables
        tables_name = tools_configuration.tables_name or \
                      registry.settings.get('onf_website.config.tables_name', 'references')
        if tables_name not in site:
            tables_manager = ReferencesManager()
            registry.notify(ObjectCreatedEvent(tables_manager))
            tables_manager.title = {'en': "References tables",
                                    'fr': "Tables de références"}
            tables_manager.short_name = {'en': "References tables",
                                         'fr': "Tables de références"}
            site[tables_name] = tables_manager
            tools_configuration.tables_name = tables_name
        else:
            tables_manager = site[tables_name]
        # check references tables
        for attr, name, factory in (('country_table_name', 'countries', CountryTable),
                                    ('region_table_name', 'regions', RegionTable),
                                    ('department_table_name', 'departments', DepartmentTable),
                                    ('territory_table_name', 'territories', TerritoryTable),
                                    ('agency_table_name', 'agencies', AgencyTable),
                                    ('unit_table_name', 'units', UnitTable),
                                    ('target_table_name', 'targets', TargetTable)):
            table_name = getattr(tools_configuration, attr, None) or \
                         registry.settings.get('onf_website.config.' + attr, name)
            if table_name not in tables_manager:
                table = factory()
                registry.notify(ObjectCreatedEvent(table))
                tables_manager[table_name] = table
                setattr(tools_configuration, attr, table_name)
        # check tools manager
        tools_name = tools_configuration.tools_name or \
                     registry.settings.get('onf_website.config.tools_name', 'tools')
        if tools_name not in site:
            tools_manager = SharedToolContainer()
            registry.notify(ObjectCreatedEvent(tools_manager))
            tools_manager.title = {'en': "Shared tools",
                                   'fr': "Outils partagés"}
            tools_manager.short_name = {'en': "Shared tools",
                                        'fr': "Outils partagés"}
            tools_manager.navigation_name = {'en': "Shared tools",
                                             'fr': "Outils partagés"}
            site[tools_name] = tools_manager
            tools_configuration.tools_name = tools_name
        else:
            tools_manager = site[tools_name]
        # check news shared tool
        news_tool_name = tools_configuration.news_tool_name or \
                         registry.settings.get('onf_website.config.news_tool_name', 'news')
        if news_tool_name not in tools_manager:
            tool = NewsManager()
            registry.notify(ObjectCreatedEvent(tool))
            tools_manager[news_tool_name] = tool
            tools_configuration.news_tool_name = news_tool_name
